"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var forOwn = require('lodash.forown');
var assign = require('lodash.assign');
var isEqual = require('lodash.isequal');
var store_1 = require('./store');
var getFromAST_1 = require('./queries/getFromAST');
var queryTransform_1 = require('./queries/queryTransform');
var printer_1 = require('graphql-tag/printer');
var readFromStore_1 = require('./data/readFromStore');
var diffAgainstStore_1 = require('./data/diffAgainstStore');
var queryPrinting_1 = require('./queryPrinting');
var batching_1 = require('./batching');
var scheduler_1 = require('./scheduler');
var Observable_1 = require('./util/Observable');
var errors_1 = require('./errors');
var ObservableQuery = (function (_super) {
    __extends(ObservableQuery, _super);
    function ObservableQuery(_a) {
        var _this = this;
        var queryManager = _a.queryManager, options = _a.options, _b = _a.shouldSubscribe, shouldSubscribe = _b === void 0 ? true : _b;
        var queryId = queryManager.generateQueryId();
        var subscriberFunction = function (observer) {
            var retQuerySubscription = {
                unsubscribe: function () {
                    queryManager.stopQuery(queryId);
                },
            };
            if (shouldSubscribe) {
                queryManager.addObservableQuery(queryId, _this);
                queryManager.addQuerySubscription(queryId, retQuerySubscription);
            }
            queryManager.startQuery(queryId, options, queryManager.queryListenerForObserver(queryId, options, observer));
            return retQuerySubscription;
        };
        _super.call(this, subscriberFunction);
        this.options = options;
        this.queryManager = queryManager;
        this.queryId = queryId;
        this.refetch = function (variables) {
            variables = variables || _this.options.variables;
            if (_this.options.noFetch) {
                throw new Error('noFetch option should not use query refetch.');
            }
            return _this.queryManager.fetchQuery(_this.queryId, assign(_this.options, {
                forceFetch: true,
                variables: variables,
            }));
        };
        this.stopPolling = function () {
            if (_this.queryManager.pollingTimers[_this.queryId]) {
                clearInterval(_this.queryManager.pollingTimers[_this.queryId]);
            }
        };
        this.startPolling = function (pollInterval) {
            if (_this.options.noFetch) {
                throw new Error('noFetch option should not use query polling.');
            }
            _this.queryManager.pollingTimers[_this.queryId] = setInterval(function () {
                var pollingOptions = assign({}, _this.options);
                pollingOptions.forceFetch = true;
                _this.queryManager.fetchQuery(_this.queryId, pollingOptions);
            }, pollInterval);
        };
    }
    ObservableQuery.prototype.result = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var subscription = _this.subscribe({
                next: function (result) {
                    resolve(result);
                    setTimeout(function () {
                        subscription.unsubscribe();
                    }, 0);
                },
                error: function (error) {
                    reject(error);
                },
            });
        });
    };
    return ObservableQuery;
}(Observable_1.Observable));
exports.ObservableQuery = ObservableQuery;
var QueryManager = (function () {
    function QueryManager(_a) {
        var _this = this;
        var networkInterface = _a.networkInterface, store = _a.store, reduxRootKey = _a.reduxRootKey, queryTransformer = _a.queryTransformer, _b = _a.shouldBatch, shouldBatch = _b === void 0 ? false : _b, _c = _a.batchInterval, batchInterval = _c === void 0 ? 10 : _c;
        this.idCounter = 0;
        this.networkInterface = networkInterface;
        this.store = store;
        this.reduxRootKey = reduxRootKey;
        this.queryTransformer = queryTransformer;
        this.pollingTimers = {};
        this.batchInterval = batchInterval;
        this.queryListeners = {};
        this.queryResults = {};
        this.scheduler = new scheduler_1.QueryScheduler({
            queryManager: this,
        });
        this.batcher = new batching_1.QueryBatcher({
            shouldBatch: shouldBatch,
            networkInterface: this.networkInterface,
        });
        this.batcher.start(this.batchInterval);
        this.fetchQueryPromises = {};
        this.observableQueries = {};
        if (this.store['subscribe']) {
            var currentStoreData_1;
            this.store['subscribe'](function () {
                var previousStoreData = currentStoreData_1 || {};
                var previousStoreHasData = Object.keys(previousStoreData).length;
                currentStoreData_1 = _this.getApolloState();
                if (isEqual(previousStoreData, currentStoreData_1) && previousStoreHasData) {
                    return;
                }
                _this.broadcastQueries();
            });
        }
    }
    QueryManager.prototype.broadcastNewStore = function (store) {
        this.broadcastQueries();
    };
    QueryManager.prototype.mutate = function (_a) {
        var _this = this;
        var mutation = _a.mutation, variables = _a.variables, _b = _a.resultBehaviors, resultBehaviors = _b === void 0 ? [] : _b, _c = _a.fragments, fragments = _c === void 0 ? [] : _c, optimisticResponse = _a.optimisticResponse, updateQueries = _a.updateQueries;
        var mutationId = this.generateQueryId();
        var mutationDef = getFromAST_1.getMutationDefinition(mutation);
        if (this.queryTransformer) {
            mutationDef = queryTransform_1.applyTransformerToOperation(mutationDef, this.queryTransformer);
            mutation = getFromAST_1.replaceOperationDefinition(mutation, mutationDef);
        }
        mutation = getFromAST_1.replaceOperationDefinition(mutation, mutationDef);
        mutation = getFromAST_1.addFragmentsToDocument(mutation, fragments);
        var mutationString = printer_1.print(mutation);
        var queryFragmentMap = getFromAST_1.createFragmentMap(getFromAST_1.getFragmentDefinitions(mutation));
        var request = {
            query: mutation,
            variables: variables,
            operationName: getFromAST_1.getOperationName(mutation),
        };
        var updateQueriesResultBehaviors = !optimisticResponse ? [] :
            this.collectResultBehaviorsFromUpdateQueries(updateQueries, { data: optimisticResponse }, true);
        this.store.dispatch({
            type: 'APOLLO_MUTATION_INIT',
            mutationString: mutationString,
            mutation: {
                id: 'ROOT_MUTATION',
                typeName: 'Mutation',
                selectionSet: mutationDef.selectionSet,
            },
            variables: variables,
            mutationId: mutationId,
            fragmentMap: queryFragmentMap,
            optimisticResponse: optimisticResponse,
            resultBehaviors: resultBehaviors.concat(updateQueriesResultBehaviors),
        });
        return this.networkInterface.query(request)
            .then(function (result) {
            _this.store.dispatch({
                type: 'APOLLO_MUTATION_RESULT',
                result: result,
                mutationId: mutationId,
                resultBehaviors: resultBehaviors.concat(_this.collectResultBehaviorsFromUpdateQueries(updateQueries, result)),
            });
            return result;
        })
            .catch(function (err) {
            _this.store.dispatch({
                type: 'APOLLO_MUTATION_ERROR',
                error: err,
                mutationId: mutationId,
            });
            return Promise.reject(err);
        });
    };
    QueryManager.prototype.queryListenerForObserver = function (queryId, options, observer) {
        var _this = this;
        return function (queryStoreValue) {
            if (!queryStoreValue) {
                return;
            }
            if (!queryStoreValue.loading || queryStoreValue.returnPartialData) {
                if (queryStoreValue.graphQLErrors || queryStoreValue.networkError) {
                    var apolloError = new errors_1.ApolloError({
                        graphQLErrors: queryStoreValue.graphQLErrors,
                        networkError: queryStoreValue.networkError,
                    });
                    if (observer.error) {
                        observer.error(apolloError);
                    }
                    else {
                        console.error('Unhandled error', apolloError, apolloError.stack);
                    }
                }
                else {
                    var resultFromStore = {
                        data: readFromStore_1.readSelectionSetFromStore({
                            store: _this.getDataWithOptimisticResults(),
                            rootId: queryStoreValue.query.id,
                            selectionSet: queryStoreValue.query.selectionSet,
                            variables: queryStoreValue.variables,
                            returnPartialData: options.returnPartialData || options.noFetch,
                            fragmentMap: queryStoreValue.fragmentMap,
                        }),
                    };
                    if (observer.next) {
                        if (_this.isDifferentResult(queryId, resultFromStore)) {
                            _this.queryResults[queryId] = resultFromStore;
                            observer.next(resultFromStore);
                        }
                    }
                }
            }
        };
    };
    QueryManager.prototype.watchQuery = function (options, shouldSubscribe) {
        if (shouldSubscribe === void 0) { shouldSubscribe = true; }
        getFromAST_1.getQueryDefinition(options.query);
        var observableQuery = new ObservableQuery({
            queryManager: this,
            options: options,
            shouldSubscribe: shouldSubscribe,
        });
        return observableQuery;
    };
    QueryManager.prototype.query = function (options) {
        var _this = this;
        if (options.returnPartialData) {
            throw new Error('returnPartialData option only supported on watchQuery.');
        }
        if (options.query.kind !== 'Document') {
            throw new Error('You must wrap the query string in a "gql" tag.');
        }
        var requestId = this.idCounter;
        var resPromise = new Promise(function (resolve, reject) {
            _this.addFetchQueryPromise(requestId, resPromise, resolve, reject);
            return _this.watchQuery(options, false).result().then(function (result) {
                _this.removeFetchQueryPromise(requestId);
                resolve(result);
            }).catch(function (error) {
                _this.removeFetchQueryPromise(requestId);
                reject(error);
            });
        });
        return resPromise;
    };
    QueryManager.prototype.fetchQuery = function (queryId, options) {
        return this.fetchQueryOverInterface(queryId, options, this.networkInterface);
    };
    QueryManager.prototype.generateQueryId = function () {
        var queryId = this.idCounter.toString();
        this.idCounter++;
        return queryId;
    };
    QueryManager.prototype.stopQueryInStore = function (queryId) {
        this.store.dispatch({
            type: 'APOLLO_QUERY_STOP',
            queryId: queryId,
        });
    };
    ;
    QueryManager.prototype.getApolloState = function () {
        return this.store.getState()[this.reduxRootKey];
    };
    QueryManager.prototype.getDataWithOptimisticResults = function () {
        return store_1.getDataWithOptimisticResults(this.getApolloState());
    };
    QueryManager.prototype.addQueryListener = function (queryId, listener) {
        this.queryListeners[queryId] = listener;
    };
    ;
    QueryManager.prototype.removeQueryListener = function (queryId) {
        delete this.queryListeners[queryId];
    };
    QueryManager.prototype.addFetchQueryPromise = function (requestId, promise, resolve, reject) {
        this.fetchQueryPromises[requestId.toString()] = { promise: promise, resolve: resolve, reject: reject };
    };
    QueryManager.prototype.removeFetchQueryPromise = function (requestId) {
        delete this.fetchQueryPromises[requestId.toString()];
    };
    QueryManager.prototype.addObservableQuery = function (queryId, observableQuery) {
        this.observableQueries[queryId] = { observableQuery: observableQuery, subscriptions: [] };
    };
    QueryManager.prototype.addQuerySubscription = function (queryId, querySubscription) {
        if (this.observableQueries.hasOwnProperty(queryId)) {
            this.observableQueries[queryId].subscriptions.push(querySubscription);
        }
        else {
            this.observableQueries[queryId] = {
                observableQuery: null,
                subscriptions: [querySubscription],
            };
        }
    };
    QueryManager.prototype.removeObservableQuery = function (queryId) {
        delete this.observableQueries[queryId];
    };
    QueryManager.prototype.resetStore = function () {
        var _this = this;
        Object.keys(this.fetchQueryPromises).forEach(function (key) {
            var reject = _this.fetchQueryPromises[key].reject;
            reject(new Error('Store reset while query was in flight.'));
        });
        this.store.dispatch({
            type: 'APOLLO_STORE_RESET',
            observableQueryIds: Object.keys(this.observableQueries),
        });
        Object.keys(this.observableQueries).forEach(function (queryId) {
            if (!_this.observableQueries[queryId].observableQuery.options.noFetch) {
                _this.observableQueries[queryId].observableQuery.refetch();
            }
        });
    };
    QueryManager.prototype.startQuery = function (queryId, options, listener) {
        var _this = this;
        this.queryListeners[queryId] = listener;
        this.fetchQuery(queryId, options);
        if (options.pollInterval) {
            if (options.noFetch) {
                throw new Error('noFetch option should not use query polling.');
            }
            this.pollingTimers[queryId] = setInterval(function () {
                var pollingOptions = assign({}, options);
                pollingOptions.forceFetch = true;
                _this.fetchQuery(queryId, pollingOptions);
            }, options.pollInterval);
        }
        return queryId;
    };
    QueryManager.prototype.stopQuery = function (queryId) {
        delete this.queryListeners[queryId];
        if (this.pollingTimers[queryId]) {
            clearInterval(this.pollingTimers[queryId]);
        }
        this.store.dispatch({
            type: 'APOLLO_QUERY_STOP',
            queryId: queryId,
        });
    };
    QueryManager.prototype.collectResultBehaviorsFromUpdateQueries = function (updateQueries, mutationResult, isOptimistic) {
        var _this = this;
        if (isOptimistic === void 0) { isOptimistic = false; }
        if (!updateQueries) {
            return [];
        }
        var resultBehaviors = [];
        var observableQueriesByName = {};
        Object.keys(this.observableQueries).forEach(function (key) {
            var observableQuery = _this.observableQueries[key].observableQuery;
            var queryName = getFromAST_1.getQueryDefinition(observableQuery.options.query).name.value;
            observableQueriesByName[queryName] =
                observableQueriesByName[queryName] || [];
            observableQueriesByName[queryName].push(observableQuery);
        });
        Object.keys(updateQueries).forEach(function (queryName) {
            var reducer = updateQueries[queryName];
            var queries = observableQueriesByName[queryName];
            if (!queries) {
                return;
            }
            queries.forEach(function (observableQuery) {
                var queryOptions = observableQuery.options;
                var queryDefinition = getFromAST_1.getQueryDefinition(queryOptions.query);
                var previousResult = readFromStore_1.readSelectionSetFromStore({
                    store: isOptimistic ? _this.getDataWithOptimisticResults() : _this.getApolloState().data,
                    rootId: 'ROOT_QUERY',
                    selectionSet: queryDefinition.selectionSet,
                    variables: queryOptions.variables,
                    returnPartialData: queryOptions.returnPartialData || queryOptions.noFetch,
                    fragmentMap: getFromAST_1.createFragmentMap(queryOptions.fragments || []),
                });
                resultBehaviors.push({
                    type: 'QUERY_RESULT',
                    newResult: reducer(previousResult, {
                        mutationResult: mutationResult,
                        queryName: queryName,
                        queryVariables: queryOptions.variables,
                    }),
                    queryOptions: queryOptions,
                });
            });
        });
        return resultBehaviors;
    };
    QueryManager.prototype.fetchQueryOverInterface = function (queryId, options, network) {
        var _this = this;
        var query = options.query, variables = options.variables, _a = options.forceFetch, forceFetch = _a === void 0 ? false : _a, _b = options.returnPartialData, returnPartialData = _b === void 0 ? false : _b, _c = options.noFetch, noFetch = _c === void 0 ? false : _c, _d = options.fragments, fragments = _d === void 0 ? [] : _d;
        var queryDef = getFromAST_1.getQueryDefinition(query);
        if (this.queryTransformer) {
            queryDef = queryTransform_1.applyTransformerToOperation(queryDef, this.queryTransformer);
        }
        var transformedQuery = getFromAST_1.replaceOperationDefinition(query, queryDef);
        transformedQuery = getFromAST_1.addFragmentsToDocument(transformedQuery, fragments);
        var queryFragmentMap = getFromAST_1.createFragmentMap(getFromAST_1.getFragmentDefinitions(transformedQuery));
        var queryString = printer_1.print(transformedQuery);
        var querySS = {
            id: 'ROOT_QUERY',
            typeName: 'Query',
            selectionSet: queryDef.selectionSet,
        };
        var minimizedQueryString = queryString;
        var minimizedQuery = querySS;
        var minimizedQueryDoc = transformedQuery;
        var initialResult;
        if (!forceFetch) {
            var _e = diffAgainstStore_1.diffSelectionSetAgainstStore({
                selectionSet: querySS.selectionSet,
                store: this.store.getState()[this.reduxRootKey].data,
                throwOnMissingField: false,
                rootId: querySS.id,
                variables: variables,
                fragmentMap: queryFragmentMap,
            }), missingSelectionSets = _e.missingSelectionSets, result = _e.result;
            initialResult = result;
            if (missingSelectionSets && missingSelectionSets.length && !noFetch) {
                var diffedQuery = queryPrinting_1.queryDocument({
                    missingSelectionSets: missingSelectionSets,
                    variableDefinitions: queryDef.variableDefinitions,
                    name: queryDef.name,
                    fragmentMap: queryFragmentMap,
                });
                var diffedQueryDef = getFromAST_1.getQueryDefinition(diffedQuery);
                minimizedQuery = {
                    id: 'ROOT_QUERY',
                    typeName: 'Query',
                    selectionSet: diffedQueryDef.selectionSet,
                };
                minimizedQueryString = printer_1.print(diffedQuery);
                minimizedQueryDoc = diffedQuery;
            }
            else {
                minimizedQuery = null;
                minimizedQueryString = null;
                minimizedQueryDoc = null;
            }
        }
        var requestId = this.generateRequestId();
        this.store.dispatch({
            type: 'APOLLO_QUERY_INIT',
            queryString: queryString,
            query: querySS,
            minimizedQueryString: minimizedQueryString,
            minimizedQuery: minimizedQuery,
            variables: variables,
            forceFetch: forceFetch,
            returnPartialData: returnPartialData || noFetch,
            queryId: queryId,
            requestId: requestId,
            fragmentMap: queryFragmentMap,
        });
        if (!minimizedQuery || returnPartialData || noFetch) {
            this.store.dispatch({
                type: 'APOLLO_QUERY_RESULT_CLIENT',
                result: {
                    data: initialResult,
                },
                variables: variables,
                query: querySS,
                complete: !!minimizedQuery,
                queryId: queryId,
            });
        }
        if (minimizedQuery) {
            var request = {
                query: minimizedQueryDoc,
                variables: variables,
                operationName: getFromAST_1.getOperationName(minimizedQueryDoc),
            };
            var fetchRequest_1 = {
                options: { query: minimizedQueryDoc, variables: variables },
                queryId: queryId,
                operationName: request.operationName,
            };
            var retPromise_1 = new Promise(function (resolve, reject) {
                _this.addFetchQueryPromise(requestId, retPromise_1, resolve, reject);
                return _this.batcher.enqueueRequest(fetchRequest_1)
                    .then(function (result) {
                    _this.store.dispatch({
                        type: 'APOLLO_QUERY_RESULT',
                        result: result,
                        queryId: queryId,
                        requestId: requestId,
                    });
                    _this.removeFetchQueryPromise(requestId);
                    if (result.errors) {
                        reject(new errors_1.ApolloError({
                            graphQLErrors: result.errors,
                        }));
                    }
                    return result;
                }).then(function () {
                    var resultFromStore;
                    try {
                        resultFromStore = readFromStore_1.readSelectionSetFromStore({
                            store: _this.getApolloState().data,
                            rootId: querySS.id,
                            selectionSet: querySS.selectionSet,
                            variables: variables,
                            returnPartialData: returnPartialData || noFetch,
                            fragmentMap: queryFragmentMap,
                        });
                    }
                    catch (e) { }
                    _this.removeFetchQueryPromise(requestId);
                    resolve({ data: resultFromStore });
                }).catch(function (error) {
                    _this.store.dispatch({
                        type: 'APOLLO_QUERY_ERROR',
                        error: error,
                        queryId: queryId,
                        requestId: requestId,
                    });
                    _this.removeFetchQueryPromise(requestId);
                    reject(new errors_1.ApolloError({
                        networkError: error,
                    }));
                });
            });
            return retPromise_1;
        }
        return new Promise(function (resolve) {
            resolve({ data: initialResult });
        });
    };
    QueryManager.prototype.isDifferentResult = function (queryId, result) {
        return !isEqual(this.queryResults[queryId], result);
    };
    QueryManager.prototype.broadcastQueries = function () {
        var queries = this.getApolloState().queries;
        forOwn(this.queryListeners, function (listener, queryId) {
            if (listener) {
                var queryStoreValue = queries[queryId];
                listener(queryStoreValue);
            }
        });
    };
    QueryManager.prototype.generateRequestId = function () {
        var requestId = this.idCounter;
        this.idCounter++;
        return requestId;
    };
    return QueryManager;
}());
exports.QueryManager = QueryManager;
//# sourceMappingURL=QueryManager.js.map